{#
  Bridge-based private network
#}
    {{ name }}.vm.network :private_network,
                  :libvirt__network_name => "{{ l.bridge }}",
                  :libvirt__forward_mode => "veryisolated",
                  :libvirt__dhcp_enabled => false,
                  :libvirt__iface_name => "vgif_{{name}}_{{l.ifindex}}",
                  :auto_config => false
