updates:
- path: openconfig-network-instance:network-instances/network-instance[name=Base]
  val:
   mpls:
    global:
     reserved-label-blocks:
      reserved-label-block:
      - local-id: "SRGB"
        config:
         local-id: "SRGB"
         lower-bound: {{ sr.srgb_range_start }}
         upper-bound: {{ sr.srgb_range_start + sr.srgb_range_size }}

# XXX currently unable to provision SIDs
# - path: openconfig-network-instance:network-instances/network-instance[name=Base]/protocols/protocol[identifier=ISIS]
#   val:
#    isis:
#     advertise-router-capability: area
#     segment-routing:
#      admin-state: enable
#      prefix-sid-range:
#       global: [null]
#      # srlb: reference mpls-labels reserved-label-block
#      tunnel-table-pref: 8 # default 11
#     interface:
#     - interface-name: system
#       ipv4-node-sid:
#        index: "{{ id }}"
#       ipv6-node-sid:
#        index: "{{ id+sr.ipv6_sid_offset }}"
