!
mpls ip
!
mpls ldp
  router-id {{ ldp.router_id }}
  transport-address interface Loopback0
  interface disabled default
  no shutdown
{% if ldp.explicit_null|default(False) %}
  label local-termination explicit-null
{% endif %}
!
interface Loopback0
  mpls ldp interface
!
{% for l in interfaces if ('ldp' in l) and not l.ldp.passive %}
interface {{ l.ifname }}
  mpls ip
  mpls ldp interface
{% endfor %}
!
