#!/bin/bash
#
set -e
#
# Create bash profile script
#
cat <<SCRIPT >/root/.bash_profile
#!/bin/bash
#
export PS1="\h(bash)$ "
SCRIPT
{% include 'linux/hosts.j2' +%}
{% if netlab_linux_distro is defined %}
{% include 'linux/'+netlab_linux_distro+'.j2' +%}
{% else %}
{% include 'linux/vanilla.j2' +%}
{% endif %}

