! {{ mpls }}
{% if ldp is defined %}
{%   include 'ios.ldp.j2' +%}
{% endif %}
{% if mpls.bgp is defined %}
{%   include 'ios.bgp-lu.j2' +%}
{% endif %}
{% if mpls.vpn is defined %}
{%   include 'ios.mplsvpn.j2' +%}
{% endif %}
