{% set pid = ospf.process|default(0) %}
{% set area = ospf.area|default("0.0.0.0") %}

updates:
- path: openconfig-network-instance:network-instances/network-instance[name=Base]/protocols
  val:
   protocol:
   - identifier: "OSPF"
     name: "0"
     config:
      identifier: "OSPF"
      name: "0"
      enabled: True

{# On 21.10.R1 this results in an error:
   MINOR: OPENCONFIG #238: configure openconfig network-instances network-instance "Base" protocols protocol OSPF name "0"
        - Configuration change failed validation - Invalid identifier. Supported values are ISIS BGP STATIC LOCAL_AGGREGATE IGMP
#}
