    {{ name }}.vm.guest = :linux
    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true

    {{ name }}.vm.provider :libvirt do |domain|
      domain.cpus = 2
      domain.memory = 1024
    end
