    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.ssh.shell = "show version"
    {{ name }}.vm.guest = :freebsd
    {{ name }}.vm.boot_timeout = 600

    {{ name }}.vm.provider :libvirt do |domain|
      domain.disk_bus = 'ide'
      domain.cpus = 2
      domain.memory = 2048
      domain.nic_model_type = "e1000"
      domain.driver = "kvm"
    end
