{#
  Forwarded VirtualBox Ports
#}

{% if defaults.providers.virtualbox.forwarded is defined %}
{%   for port,base in defaults.providers.virtualbox.forwarded.items() %}
    {{ name }}.vm.network "forwarded_port",
      guest: {{ defaults.ports[port] }}, host: {{ base + n.id }}, id: "{{ port }}"
{%   endfor %}
{% endif %}
