    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.ssh.username = "root"
    {{ name }}.ssh.password = 'arrcus'

    {{ name }}.vm.provider :libvirt do |domain|
      domain.cpus = 2
      domain.memory = 4096
      domain.driver = "kvm"
    end
