{% set KW_NETWORK_TYPE = {'point-to-point': 'p2p','point-to-multipoint': 'p2mp', 'non-broadcast': 'nbma' } %}
{% set pid = ospf.process|default(1) %}

{% if ospf.router_id|ipv4 %}
routing-options {
  router-id {{ ospf.router_id }}
}
{% endif %}

{% if ospf.af.ipv4 is defined %}
{% include 'junos.ospfv2.j2' %}
{% endif %}
{% if ospf.af.ipv6 is defined %}
{% include 'junos.ospfv3.j2' %}
{% endif %}
