    {{ name }}.vm.synced_folder ".", "/vagrant", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.vm.boot_timeout = 360
    {{ name }}.vm.guest = :freebsd

    {{ name }}.ssh.shell = "run bash"
    {{ name }}.vm.provider :virtualbox do |domain|
      domain.cpus = 2
      domain.memory = 6144
    end
