{#
  Point-to-point network using UDP tunnel
#}
    {{ name }}.vm.network :private_network,
                  :libvirt__tunnel_type => "udp",
                  :libvirt__tunnel_local_ip => "127.1.{{ n.id }}.{{ l.ifindex }}",
                  :libvirt__tunnel_local_port => "{{ 10000 + l.ifindex}}",
                  :libvirt__tunnel_ip => "127.1.{{ l.remote_id}}.{{ l.remote_ifindex }}",
                  :libvirt__tunnel_port => "{{ 10000 + l.remote_ifindex}}",
                  :libvirt__iface_name => "vgif_{{name}}_{{l.ifindex}}",
                  auto_config: false
