    cwd = Dir.pwd.split("/").last
    guest_name = "{{ name }}"
    {{ name }}.vm.guest = :tinycore
    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true

    {{ name }}.ssh.insert_key = false

    # Limit CPU once Box has finished booting.
    {{ name }}.trigger.after :up do |trigger|
      trigger.name = "Finished Message"
      trigger.info = "Machine is up! : #{cwd}_#{guest_name}"
      trigger.run = {inline: "virsh schedinfo #{cwd}_#{guest_name} --set vcpu_quota=35000"}
    end

    {{ name }}.vm.provider :libvirt do |domain|
      domain.default_prefix = "#{cwd}_"
      domain.cpus = 2
      domain.memory = 4096
      domain.disk_bus = "ide"
      {% if "amd" in defaults.processor|lower %}
      domain.cpu_mode = "custom"
      {% endif %}
    end
