{% set pid = ospf.process|default(1) %}
{% if ospf.af.ipv4 is defined %}
{% include 'dellos10.ospfv2.j2' %}
{% endif %}
{% if ospf.af.ipv6 is defined %}
{% include 'dellos10.ospfv3.j2' %}
{% endif %}
