mpls ip
!
segment-routing mpls
 !
 set-attributes
  address-family ipv4
   explicit-null
 !
 connected-prefix-sid-map
{% if 'ipv4' in loopback %}
  address-family ipv4
   {{ loopback.ipv4 }} index {{ id }} range 1
{% endif %}
!
router isis Gandalf
 segment-routing mpls
