{% for vname,vdata in vrfs.items() %}
vrf instance {{ vname }}
 rd {{ vdata.rd }}
!
{% if 'ipv4' in vdata.af %}
ip routing vrf {{ vname }}
!
{% endif %}
{% if 'ipv6' in vdata.af %}
ipv6 routing vrf {{ vname }}
!
{% endif %}
{% endfor %}
