[defaults]
ansible_managed = # Ansible Managed File
filter_plugins = ./filter_plugins
forks=10
gathering=explicit
host_key_checking=False
interpreter_python=auto_silent
inventory={{ inventory }}
retry_files_enabled=false
roles_path=./roles
ssh_args="-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null"
stdout_callback=yaml

[persistent_connection]
command_timeout=60

[paramiko_connection]
host_key_auto_add=no
host_key_checking=no

