    if VAGRANT_COMMAND == "ssh" or VAGRANT_COMMAND == "scp"
        config.ssh.username = 'cumulus'
    end

    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true

    {{ name }}.vm.provider :libvirt do |domain|
      domain.cpus = 2
      domain.memory = 1024
    end

    # Run cumulus-specific Vagrant box configuration
    {{ name }}.vm.provision :shell , :inline => $cumulus_script