    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true
    {{ name }}.ssh.insert_key = false

    {{ name }}.vm.provider :libvirt do |domain|
      domain.disk_bus = 'ide'
      domain.cpus = 1
      domain.memory = 1024
    end
