{% for vname,vdata in vrfs.items() %}
vrf definition {{ vname }}
 rd {{ vdata.rd }}
{%   for rt in vdata.import %}
 route-target import {{ rt }}
{%   endfor %}
{%   for rt in vdata.export %}
 route-target export {{ rt }}
{%   endfor %}
!
{%   for af in ['ipv4','ipv6'] if af in vdata.af %}
 address-family {{ af }}
 exit-address-family
!
{%   endfor %}
{% endfor %}
