    {{ name }}.vm.guest = :freebsd
    {{ name }}.ssh.insert_key = false
    {{ name }}.ssh.shell = "/ip address print"
    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true

    {{ name }}.vm.provider :libvirt do |domain|
      domain.cpus = 1
      domain.memory = 256
    end
