    {{ name }}.vm.synced_folder ".", "/vagrant", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.ssh.username = "root"
    {{ name }}.ssh.password = 'arrcus'
    {{ name }}.vm.provider :virtualbox do |vb|
      vb.cpus = 2
      vb.memory = 4096
    end
