{% if ldp is defined %}
{%   include 'eos.ldp.j2' +%}
{% endif %}
{% if mpls.bgp is defined %}
{%   include 'eos.bgp-lu.j2' +%}
{% endif %}
{% if mpls.vpn is defined %}
{%   include 'eos.mplsvpn.j2' +%}
{% endif %}
