# Generated by evpn sros.gnmi.j2 template
updates:
- path: configure/router[router-name=Base]
  val:
   bgp:
    group:
    - group-name: {{ 'ibgp-ipv4' if evpn.use_ibgp else 'ebgp' }} # hardcoded
      family:
       evpn: True
{%     set bgp_type='ibgp' if evpn.use_ibgp else 'ebgp' %}
       ipv4: {{ 'ipv4' in bgp.address_families[bgp_type] and not evpn.use_ibgp }}
       ipv6: {{ 'ipv6' in bgp.address_families[bgp_type] and not evpn.use_ibgp }}
