system hostname {{ inventory_hostname }}
!
!
interface loopback0
 enable true
 subinterface 0
   {% if 'ipv4' in loopback %}
   ipv4 address {{ loopback.ipv4 | ansible.netcommon.ipv4('address') }} prefix-length 32
   {% endif %}
   {% if 'ipv6' in loopback %}
   ipv6 address {{ loopback.ipv6 | ansible.netcommon.ipv6('address') }} prefix-length 64
   {% endif %}
!
!
!
{% for l in interfaces|default([]) %}
interface {{ l.ifname }}
 enable true
 subinterface 0
   {% if 'ipv4' in l %}
   ipv4 address {{ l.ipv4 | ansible.netcommon.ipv4('address') }} prefix-length {{ l.ipv4 | ansible.netcommon.ipaddr('prefix') }}
   {% endif %}
   {% if 'ipv6' in l %}
   ipv6 address {{ l.ipv6 | ansible.netcommon.ipv6('address') }} prefix-length {{ l.ipv6 | ansible.netcommon.ipaddr('prefix') }}
   {% endif %}
!
{% endfor %}

no lldp interface ma1
