{#
  Bridge-based private network
#}

    {{ name }}.vm.network "private_network",
      virtualbox__intnet: "{{ l.bridge | default('net_'+l.linkindex|string) }}", ip: "169.254.1.{{ n.id + 100 }}", auto_config: false
