    {{ name }}.vm.synced_folder ".", "/vagrant", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.ssh.shell = "run bash"
    {{ name }}.vm.boot_timeout = 360
    {{ name }}.vm.guest = :freebsd

    {{ name }}.vm.provider :libvirt do |domain|
      domain.cpus = 2
      domain.features = ['acpi']
      domain.loader = "/usr/share/OVMF/OVMF_CODE.fd"
      domain.memory = 6144
      domain.disk_bus = "sata"
      domain.disk_device = "sda"
      domain.disk_driver :cache => "none"
      domain.nic_model_type = "e1000"
      domain.graphics_type = "none"
      domain.driver = "kvm"
      {% if "amd" in defaults.processor|lower %}
      domain.cpu_mode = "custom"
      {% endif %}
    end
